var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var errorMsgs = {
  actionsList: 'The actions list must be an array of strings',
  actionType: function actionType(type) {
    return 'The action "' + type + '" is not part of the provided actions list';
  },
  dispatch: '"dispatch" must be a function',
  meta: 'The action metadata must be an object',
  middleware: 'The middleware list must be an array of functions'
};

function validateMeta(meta) {
  if ((typeof meta === 'undefined' ? 'undefined' : _typeof(meta)) !== 'object' || meta === null || Array.isArray(meta)) throw new Error(errorMsgs.meta);
  return true;
}

export default (function (actionsList, dispatch, middleware) {
  if (!Array.isArray(actionsList)) throw new Error(errorMsgs.actionsList);
  var actionsMap = {};
  actionsList.forEach(function (type) {
    if (typeof type !== 'string') throw new Error(errorMsgs.actionsList);
    actionsMap[type] = type;
  });
  function validateType(type) {
    if (actionsMap[type]) return type;else throw new Error(errorMsgs.actionType(type));
  }
  if (typeof dispatch !== 'function') throw new Error(errorMsgs.dispatch);
  if (middleware !== undefined) {
    if (!Array.isArray(middleware)) throw new Error(errorMsgs.middleware);
    middleware.reverse();
    middleware.forEach(function (func) {
      if (typeof func !== 'function') throw new Error(errorMsgs.middleware);
      var nextDispatch = dispatch;
      dispatch = function dispatch(action) {
        return func(action, function () {
          return nextDispatch(action);
        }, validateType);
      };
    });
  }
  function dispatcher(type, data, meta) {
    validateType(type);
    if (meta !== undefined) {
      validateMeta(meta);
      return dispatch({ type: type, data: data, meta: _extends({}, meta) });
    }
    return dispatch({ type: type, data: data, meta: {} });
  }
  return dispatcher;
});
